;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;This script used to plot e3sm diagnostics for TCs;;;;;;
;TC Basin definitions.(Knutson et. al.,2020,BAMS)
;North Atlantic: 0-90N, 265-360E
;Northeast Pacific: 0-90N, 180-265E
;Northwest Pacific: 0-90N, 100-180E
;North Indian: 0-90N, 30-100E
;South Indian: 90S-0, 20-135E
;Southwest Pacific: 90S-0, 135-295E
;South Atlantic(SA1+SA2): 90S-0, South America (295-360E) to Africa (0-20E)
;The North Atlantic-northeast Pacific boundary is on a
;diagonal tracing a path through Mexico and Central America.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_code.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_csm.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/contributed.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/esmf/ESMF_regridding.ncl"

begin

  sgrd      = "8x8"
  sgrp      = "all" ;"p990"
  region    = "SH"
  latN      = 0.0
  latS      = -90.0
  cenlon    =  0
  figkey    = "trkdens_respons_" + sgrd +"_"+sgrp +"_" +region
  datdir    = "/anvil/projects/x-phy220062/x-szhang3/evaluation/etc_statistics/etc_info"+"_" + sgrd +"/" + sgrp
  Figstr    = (/"(a)", "(b)", "(a)", "(b)", "(c)", "(d)"/)
  ystr      = 1979
  yend      = 2014
  nyr       = yend - ystr +1
  cbstr     = "Response (~F33~D~N~~F~) of track density (storms/year/"+sgrd+" degree box)"

  Varnam    = "trkdens"
  Varstr    = "Track density"
  Varunt    = "Storm counts per year per 8x8 square degree"

  ;;;;;;;;;;select the regions to calculate the rmse;;;;;;;;;;;;;
  Groups    = (/"CLIM30", "CLIM30ML","SSP245", "SSP245ML","SSP585", "SSP585ML" /)
  LabList   = (/"CLIM",   "ML(CLIM)",\
                "SSP245 - CLIM", "ML(SSP245) - ML(CLIM)", \
                "SSP585 - CLIM", "ML(SSP585) - ML(CLIM)" /)
  ngrps     = dimsizes(Groups)
  timtag    = Groups
  timtag    = ystr+"-"+yend
  ngrps     = dimsizes(Groups)
  timtag    = Groups
  timtag    = ystr+"-"+yend
  LabList   = LabList + " (" +region+")"

  sset    = "set1"
  
  do i = 0,ngrps-1,1

   fna  = datdir+"/"+Groups(i)+"_"+sset+"_Track_statistics_"+timtag(i)+"_"+sgrd+".nc"
   fla  = addfile(fna, "r")
   lat  = fla->lat
   lon  = fla->lon
   time = fla->time 
   nytm = toint(str_get_cols(timtag(i),5,8)) - toint(str_get_cols(timtag(i),0,3)) + 1

  ;calculate the clime mean of annual total  
   if(i.eq.0) then 
     vall      = new((/ngrps,nyr,dimsizes(lat),dimsizes(lon)/),typeof(fla->$Varnam$))
     vall!0    = "group"
     vall!1    = "time"
     vall!2    = "lat"
     vall!3    = "lon"
     vall&lat  = lat
     vall&lon  = lon
   end if 
   var0     = reshape(fla->$Varnam$,(/nytm,12,dimsizes(lat),dimsizes(lon)/))
   vall(i,:,:,:) = dim_sum_n(var0,1)
   delete([/var0,lat,lon,time,nytm/])
   print(" ")
   print("........................... ")
   print("igp : " + i) 
   print("........................... ")
  end do 

  do i = 0,ngrps-1,1
    if(Groups(i).eq."SSP245".or.Groups(i).eq."SSP585") then
      indref = ind(Groups.eq."CLIM30")
      vref   = vall(indref,:,:,:)
    else
      indref = ind(Groups.eq."CLIM30ML")
      vref   = vall(indref,:,:,:)
    end if
    vart = vall(i,:,:,:)

   ;wrf_smooth_2d(vart,3)
   ;wrf_smooth_2d(vref,3)

   ;vart = smth9_Wrap(vart, 0.50, -0.25, False) 
   ;vref = smth9_Wrap(vref, 0.50, -0.25, False)

    vart = smth9_Wrap(vart, 0.50, 0.25, True)
    vref = smth9_Wrap(vref, 0.50, 0.25, True)

    var  = dim_avg_n_Wrap(vart,0) 
    var  = var - dim_avg_n(vref,0)

    ;;student-t test
    sigr  = 0.10                                 ; critical sig lvl for r
    xtmp  = vref(lat|:,lon|:,time|:)
    ytmp  = vart(lat|:,lon|:,time|:)
    xAve  = dim_avg (xtmp)              ; calculate means at each grid point
    yAve  = dim_avg (ytmp)
    xVar  = dim_variance (xtmp)         ; calculate variances
    yVar  = dim_variance (ytmp)
    xEqv  = equiv_sample_size (xtmp, sigr,0)
    yEqv  = equiv_sample_size (ytmp, sigr,0)

    xN    = dimsizes(vref(:,0,0)) 
    yN    = dimsizes(vart(:,0,0)) 
    iflag = False                        ; population variance similar
    prob  = var
    prob  = (/ttest(xAve,xVar,xN,yAve,yVar,yN,iflag, False)/)
    print(xN+ " " + yN)
    printMinMax(prob,False)
    prob  = where(prob.lt.sigr,prob,prob@_FillValue)
    plot_sig_test = True
 
   lstr =  Figstr(i) + "  " +LabList(i)
   rstr =  "~F33~D~N~~F~ETC (#/Year): " + sprintf("%.1f",sum(var))

   ;----------------------------------------------------------------------
   ; Plotting section
   ;----------------------------------------------------------------------

   if(i.eq.0)then
    ;;;;;;;open a wks for plot;;;;;;;;;;;;;;;;;;;
    wtype = "pdf"
    wtype@wkPaperSize  = "A4"
   ;wtype@wkOrientation   = "landscape" ;;pictures displayed along the length
 
    plotname = "etc_track_" + figkey 
    wks1     = gsn_open_wks(wtype,plotname)
    plot     = new (ngrps, graphic)

    print(" ")
    print("........................... ")
    print("plot : " + plotname+".pdf")
    print("........................... ")

   end if 

   ;;;set up the plot information;;;;
   lndcolor   = "Black"
   if(sgrd.eq."8x8") then 
     cnlevels  := (/0.1,1,5,10,15,20,25,30,35,40,45,50/)
     cnlevel2  := fspan(-3,3,21) ;(/-4,-3,-2,-1,-0.5,-0.1,0.1,0.2,1,2,3,4/)
     print(cnlevel2)
     cnlevel2(9) = -0.1
     cnlevel2(11) =  0.1
     cmap2     := read_colormap_file("MPL_bwr")
     cmap2(dimsizes(cmap2(:,0))/2-5:dimsizes(cmap2(:,0))/2+5,:) = 0
   else
     cnlevels  := (/0.1,1,2,4,6,8,10,12,14,16,18/)
   end if 

;---Resources to share between both plots
   res                = True     ; Plot modes desired.
   res@gsnDraw        = False
   res@gsnFrame       = False
   res@gsnMaximize    = False     ; Maximize plot
   res@tmXTOn         = False
   res@tmYROn         = False

   BorderThick = 0.5
   res@tmBorderThicknessF    = BorderThick
   res@tmXBMajorThicknessF   = BorderThick
   res@tmXBMinorThicknessF   = BorderThick*0.5
   res@tmYLMajorThicknessF   = BorderThick
   res@tmYLMinorThicknessF   = BorderThick*0.5
   res@tmXTMajorThicknessF   = BorderThick
   res@tmXTMinorThicknessF   = BorderThick*0.5
   res@tmYRMajorThicknessF   = BorderThick
   res@tmYRMinorThicknessF   = BorderThick*0.5

   FontHeightF = 0.020
   res@tiMainFontHeightF    = FontHeightF
   res@tmYLLabelFontHeightF = FontHeightF
   res@tmXBLabelFontHeightF = FontHeightF

   mpres     = res
   resx      = res
   resx1     = res

   res@gsnStringFontHeightF  = FontHeightF*1.2
   res@lbLabelFontHeightF    = FontHeightF*1.0
   res@lbTitleFontHeightF    = FontHeightF*1.0

   res@cnFillOn              = True     ; color plot desired
   res@cnLinesOn             = False    ; turn off contour lines
   res@cnLineLabelsOn        = False    ; turn off contour labels
  ;res@cnFillMode            = "CellFill";"RasterFill"   ; turn raster on
   res@cnMissingValFillColor = "White"

   res@lbLabelBarOn             = False   ; Will turn on in panel later
   res@pmLabelBarParallelPosF   = 0.50 ;61
   res@pmLabelBarOrthogonalPosF = 0.05
  ;res@lbOrientation            = "Horizonal" ; "vertical"  ; vertical label bar
   res@pmLabelBarHeightF        = 0.06 ;0.60
   res@pmLabelBarWidthF         = 0.60 ;0.06
   res@lbTitleString            = ""
   res@lbTitlePosition          = "Top" ;"Right"         ; title location
  ;res@lbTitleDirection         = "Across"        ; letter angle
  ;res@lbTitleAngleF            = 90.          ; title angle
   res@lbBoxLinesOn             = True
  ;res@lbBoxSeparatorLinesOn    = False
  ;res@lbLabelStride            = 15

   res@gsnCenterString    = ""
   res@gsnLeftString      = ""
   res@gsnRightString     = "" 
   res@tiMainString       = "" 

;---Control appearance of map.
   mpres@gsnPolar               = region                           ; specify the hemisphere
   mpres@gsnPolarLabelFontHeightF= FontHeightF
   mpres@mpMaxLatF              = latN
   mpres@mpMinLatF              = latS
   mpres@gsnMajorLatSpacing     = 30
   mpres@gsnMajorLonSpacing     = 45
   mpres@gsnMinorLatSpacing     = 15
   mpres@gsnMinorLonSpacing     = 15
   mpres@mpLabelsOn             = False
   mpres@mpPerimOn              = False
   mpres@mpFillOn               = False
   mpres@mpGeophysicalLineColor = lndcolor  ; color of cont. outlines
   mpres@mpGeophysicalLineThicknessF = 1.0          ; thickness of outlines
   mpres@mpOutlineOn            = True
  ;mpres@mpOutlineDrawOrder     = "PostDraw"
  ;mpres@mpFillDrawOrder        = "Predraw"
  ;mpres@mpGridAndLimbOn        = True
  ;mpres@mpGridLatSpacingF      = 90.0
  ;mpres@mpGridLonSpacingF      = 180.0
   mpres@mpGridLineThicknessF   = 1.0
  ;mpres@mpGridAndLimbDrawOrder="PreDraw"
  ;mpres@mpMinLonF              = 0  ;-180.
  ;mpres@mpMaxLonF              = 360;180.
   mpres@mpCenterLonF           = cenlon 
   mpres@gsnPolarLabelSpacing   = 60  

   mpres@tmYLLabelsOn     = True
   mpres@tmYLOn           = True
   mpres@tmYROn           = False

   plot(i) = gsn_csm_map(wks1,mpres)

;---Resources for plotting original data
   res1=res
   res1@cnLevelSelectionMode        = "ExplicitLevels"  
  ;res1@cnFillPalette               = "ViBlGrWhYeOrRe";"testcmap" ;"NCV_jaisnd";"temp_19lev"
   res1@cnLinesOn                   = False ;plot_contour  
   res1@cnLineLabelsOn              = False  
   res1@cnInfoLabelOn               = False
   res1@cnLineLabelDensityF         = 2.0
   res1@cnLineLabelFontHeightF      = 0.020
   res1@cnLineLabelBackgroundColor  = -1
   res1@gsnContourLineThicknessesScale = 1.0
   res1@cnLineThicknessF            = 0.001
   res1@cnLineColor                 = "Black"
   res1@gsnContourZeroLineThicknessF= 0.2
   res1@gsnContourNegLineDashPattern= 0
   res1@cnSmoothingOn               = True      
   res1@gsnAddCyclic                = True
   res1@gsnLeftString               = "" ;lstr 
   res1@gsnLeftStringOrthogonalPosF = 0.04
   res1@gsnRightString              = "" ;rstr
   res1@gsnRightStringOrthogonalPosF= 0.04
   res1@gsnCenterString               = lstr
   res1@gsnCenterStringOrthogonalPosF = 0.08
   if(i.eq.0) then 
     res1@cnFillPalette               = "wh-bl-gr-ye-re";"precip2_17lev"
     res1@cnLevels                   := cnlevels
     res1@lbTitleString               = "("+Varunt+")";
   else
     res1@cnFillPalette               = cmap2;"temp_diff_18lev" ;"NCV_blue_red" ;"NCV_blu_red";"CBR_coldhot";"precip2_17lev"
     res1@cnLevels                   := cnlevel2
     res1@lbTitleString               = "("+Varunt+")";
   end if 
  ;wrf_smooth_2d(var,3)
   plot_org1  = gsn_csm_contour(wks1,var,res1)
   overlay(plot(i) ,plot_org1)

   if(plot_sig_test) then
     res21 = res
     res21@gsnLeftString                = ""
     res21@gsnRightString               = ""
     res21@gsnCenterString              = ""
     res21@cnMissingValFillColor        = "Transparent"
     res21@cnFillOn                     = True
     res21@cnLinesOn                    = False
     res21@cnLineLabelsOn               = False       ; turn off contour labels
     res21@cnInfoLabelOn                = False
     res21@cnMonoFillColor              = True        ; default color is fground [black]
     res21@cnMonoFillPattern            = True
     res21@cnMonoFillScale              = True
     res21@cnFillPattern                = 17
     res21@cnFillScaleF                 = 0.8
     res21@cnFillColor                  = "Black"
     res21@cnFillDotSizeF               = 0.003
     res21@lbLabelBarOn                 = False
     plot_cont2 = gsn_csm_contour(wks1,prob,res21)
     overlay(plot(i),plot_cont2)
     delete([/res21/])
   end if

 delete([/res,res1,mpres,var/])

end do ; end of experiment group loop

;---Draw both plots in a panel
;***************************************
; panel first set of plots
;***************************************
  pres1                          = True
  pres1@gsnFrame                 = True      ; don't advance frame yet
  pres1@gsnMaximize              = False
  pres1@gsnPanelLabelBar         = True
  pres1@pmLabelBarParallelPosF   = 0.0
  pres1@pmLabelBarOrthogonalPosF = -0.02
 ;pres1@lbOrientation            = "vertical"  ; vertical label bar
  pres1@pmLabelBarHeightF        = 0.05 ;0.21
  pres1@pmLabelBarWidthF         = 0.75 ;0.05
  pres1@lbLabelFontHeightF       = FontHeightF * 0.6
  pres1@lbTitleFontHeightF       = FontHeightF * 0.65
  pres1@lbTitleString            = cbstr 
  pres1@lbTitlePosition          = "Top" ;"Right"         ; title location
 ;pres1@lbTitleDirection         = "Across"        ; letter angle
 ;pres1@lbTitleAngleF            = 90.          ; title angle
  pres1@lbBoxLinesOn             = True
 ;pres1@lbBoxSeparatorLinesOn    = False
 ;pres1@lbLabelStride            = 15

  pres1@gsnPanelMainString       = " "
  pres1@gsnPanelMainFontColor    = "Black"
  pres1@gsnPanelMainFontHeightF  = 0.028
  pres1@gsnPanelXWhiteSpacePercent = 5
  gsn_panel(wks1,(/plot(2:)/),(/2,2/),pres1)

end
